package cn.iocoder.yudao.module.member.api.score;

import cn.iocoder.yudao.framework.redis.helper.RedisDistributedLock;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreDetailUpdateReqDto;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateRespDTO;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreUpdateReqDTO;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.service.memberUserScore.MemberUserScoreService;
import cn.iocoder.yudao.module.member.service.memberUserScoreDetail.MemberUserScoreDetailService;
import cn.iocoder.yudao.module.member.service.memberUserScoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.vo.memberUserScoreLog.MemberUserScoreLogCreateReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
@Slf4j
@Service
public class MemberUserScoreApiImpl implements MemberUserScoreApi{

    @Resource
    private MemberUserScoreLogService logService;
    @Resource
    private MemberUserScoreDetailService scoreDetailService;
    @Resource
    private MemberUserScoreService memberUserScoreService;
    @Resource
    private RedisDistributedLock redisDistributedLock;
    @Override
    @Transactional(rollbackFor = Exception.class)
    public MemberUserScoreOperateRespDTO operateScore(MemberUserScoreOperateReqDTO req) {
        //todo 校验
        String lockKey = "member:operate:socre:" + req.getMemberId();
        boolean lock = redisDistributedLock.lock(lockKey, 5000, 3, 100);
        if (!lock) {
            return MemberUserScoreOperateRespDTO.builder().success(false).code(500).reqDTO(req).build();
        }
        try {
            Long logId = saveScoreLog(req);
            saveScoreDetail(req, logId);
            updateUserScore(req);
        } finally {
            redisDistributedLock.releaseLock(lockKey);
        }
        return MemberUserScoreOperateRespDTO.success(req);
    }

    private void updateUserScore(MemberUserScoreOperateReqDTO req) {
        memberUserScoreService.updateUserScore(MemberUserScoreUpdateReqDTO.builder()
                        .memberId(req.getMemberId())
                        .scoreCount(req.getScoreCount())
                        .operateType(req.getSourceType() == ScoreSourceTypeEnum.MANUAL_OPERATE ?
                                req.getOperateType() : req.getSourceType().getOperateType())
                        .sourceType(req.getSourceType())
                .build());
    }

    private void saveScoreDetail(MemberUserScoreOperateReqDTO req, Long scoreLogId) {
        scoreDetailService.updateScoreDetail(MemberUserScoreDetailUpdateReqDto.builder()
                        .memberId(req.getMemberId())
                        .scoreCount(req.getScoreCount())
                        .operateType(req.getSourceType() == ScoreSourceTypeEnum.MANUAL_OPERATE ?
                                req.getOperateType() : req.getSourceType().getOperateType())
                        .sourceType(req.getSourceType())
                        .scoreLogId(scoreLogId)
                        .expireDays(req.getExpireDays())
                .build());
    }

    private Long saveScoreLog(MemberUserScoreOperateReqDTO req) {
        return logService.createScoreLog(MemberUserScoreLogCreateReq.builder()
                .memberId(req.getMemberId())
                .scoreCount(req.getScoreCount())
                .operateType(req.getSourceType() == ScoreSourceTypeEnum.MANUAL_OPERATE ?
                        req.getOperateType().getValue() : req.getSourceType().getOperateType().getValue())
                .sourceType(req.getSourceType().getValue())
                .extParam(req.getExtParam())
                .build());
    }
}
